<?php
/**
 * Cardinity Payment Plugin for Boxbilling
 *
 * @copyright Copyright (C) 2015 Cations Web Dev
 * @license   http://www.gnu.org/licenses/gpl-2.0.html GNU/GPL
 *
 */

require_once("bb-library/Payment/Adapter/cardinity/vendor/autoload.php");

use Cardinity\Client;
use Cardinity\Method\Payment;
use Cardinity\Exception;

class Payment_Adapter_Cardinity implements \Box\InjectionAwareInterface
{
    private $config = array();

    protected $di;

    public function setDi($di)
    {
        $this->di = $di;
    }

    public function getDi()
    {
        return $this->di;
    }

    public function __construct($config)
    {
        $this->config = $config;

        if (!isset($this->config['consumerKey'])) {
            throw new Payment_Exception('Payment gateway "Cardinity" is not configured properly. Please update configuration parameter "Consumer Key" at "Configuration -> Payments".');
        }

        if (!isset($this->config['consumerSecret'])) {
            throw new Payment_Exception('Payment gateway "Cardinity" is not configured properly. Please update configuration parameter "Consumer Secret" at "Configuration -> Payments".');
        }
    }

    public static function getConfig()
    {
        return array(
            'supports_one_time_payments' => true,
            'description' => 'To setup Cardinity merchant account in BoxBilling you need to go to <i>Dashboard &gt; Integration &gt; API Settings</i> and copy Consumer Key and Consumer Secret and paste here in form fields.',
            'form' => array(
                'consumerKey' => array('text', array(
                    'label' => 'Consumer Key',
                    'description' => 'Cardinity Consumer Key',
                    ),
                ),
                'consumerSecret' => array('text', array(
                    'label' => 'Consumer Secret',
                    'description' => 'Cardinity Consumer Secret',
                    ),
                ),
                'testConsumerKey' => array('text', array(
                    'label' => 'Test Consumer Key',
                    'description' => 'Cardinity Test Consumer Key',
                    ),
                ),
                'testConsumerSecret' => array('text', array(
                    'label' => 'Test Consumer Secret',
                    'description' => 'Cardinity Test Consumer Secret',
                    ),
                ),
            ),
        );
    }

    public function getHtml($api_admin, $invoice_id, $subscription)
    {
        $invoiceModel = $this->di['db']->load('Invoice', $invoice_id);

        return $this->_generateForm($invoiceModel);
    }

    public function getAmount(\Model_Invoice $invoice)
    {
        $invoiceService = $this->di['mod_service']('Invoice');
        return (float)$invoiceService->getTotalWithTax($invoice);
    }

    public function getInvoiceTitle(\Model_Invoice $invoice)
    {
        $invoiceItems = $this->di['db']->getAll('SELECT title from invoice_item WHERE invoice_id = :invoice_id', array(':invoice_id' => $invoice->id));

        $params = array(
            ':id' => sprintf('%05s', $invoice->nr),
            ':serie' => $invoice->serie,
            ':title' => $invoiceItems[0]['title']);
        $title = __('Payment for invoice :serie:id [:title]', $params);
        if (count($invoiceItems) > 1) {
            $title = __('Payment for invoice :serie:id', $params);
        }
        return $title;
    }

    public function logError(Exception $e, Model_Transaction $tx)
    {
        $body = $e->getJsonBody();
        $err = $body['error'];
        $tx->txn_status = $err ['type'];
        $tx->error = $err['message'];
        $tx->status = 'processed';
        $tx->updated_at = date('Y-m-d H:i:s');
        $this->di['db']->store($tx);


        if ($this->di['config']['debug']) {
            error_log(json_encode($e->getJsonBody()));
        }
        throw new Exception($tx->error);
    }

    public function processTransaction($api_admin, $id, $data, $gateway_id)
    {
        $invoice = $this->di['db']->getExistingModelById('Invoice', $data['get']['bb_invoice_id']);
        $tx = $this->di['db']->getExistingModelById('Transaction', $id);

        $invoiceAmountInCents = $this->getAmount($invoice);

        $consumerKey = $this->config['consumerKey'];
        if ($this->config['test_mode']) {
            $consumerKey = $this->get_test_consumerKey();
        }

        $consumerSecret = $this->config['consumerSecret'];
        if ($this->config['test_mode']) {
            $consumerSecret = $this->get_test_consumerSecret();
        }


        $title = $this->getInvoiceTitle($invoice);
        $tx->invoice_id = $invoice->id;

        $client = Client::create([
            'consumerKey' => $consumerKey,
            'consumerSecret' => $consumerSecret,
        ]);


        if (isset($_REQUEST['3dreturn']) && $_REQUEST['3dreturn'] == "3d_secure_cardinity") {
            $method = new Payment\Finalize($_SESSION["lastOrderId"], $_REQUEST['PaRes']);
            $payment = $client->call($method);
            $status = $payment->getStatus(); // value will be 'approved'
        } else {
            $method = new Payment\Create([
                'amount' => (float)$invoiceAmountInCents,
                'currency' => $invoice->currency,
                'settle' => true,
                'description' => $title,
                /*'description' => "3d-fail",*/
                'order_id' => $tx->invoice_id,
                'country' => $invoice->buyer_country,
                'payment_method' => 'card',
                'payment_instrument' => [
                    'pan' => $data['post']['cc_number'],
                    'exp_year' => (int)$data['post']['ex_year'],
                    'exp_month' => (int)$data['post']['ex_month'],
                    'cvc' => $data['post']['cvv'],
                    'holder' => $data['post']['holder'],
                ],
            ]);

            try {
                /** @type Cardinity\Method\Payment\Payment */
                $payment = $client->call($method);
                $status = $payment->getStatus(); // value will be 'approved'
            } catch (Exception\Declined $exception) {
                /** @type Cardinity\Method\Payment\Payment */
                $payment = $exception->getResult();
                $status = $payment->getStatus(); // value will be 'declined'
                $errors = $exception->getErrors(); // list of errors occured
            }
        }

        if ($payment->getStatus() == "approved") {
            $tx->txn_status = $status;
            $tx->txn_id = $payment->getId();
            $tx->amount = $payment->getAmount();
            $tx->currency = $payment->getCurrency();

            $bd = array(
                'amount' => $tx->amount,
                'description' => 'Cardinity transaction ' . $payment->getId(),
                'type' => 'transaction',
                'rel_id' => $tx->id,
            );

            $client = $this->di['db']->getExistingModelById('Client', $invoice->client_id);

            $clientService = $this->di['mod_service']('client');
            $clientService->addFunds($client, $bd['amount'], $bd['description'], $bd);

            $invoiceService = $this->di['mod_service']('Invoice');
            if ($tx->invoice_id) {
                $invoiceService->payInvoiceWithCredits($invoice);
            }
            $invoiceService->doBatchPayWithCredits(array('client_id' => $client->id));
        } elseif ($payment->getStatus() == "pending") {
            $_SESSION["lastOrderId"] = $payment->getId();
            $authData = $payment->getAuthorizationInformation();
            $configPath = BB_PATH_ROOT . '/bb-config.php';
            $config = include $configPath;
            $returnObj = '<html>
                   <head>
                      <title>Redirecting to 3-D Secure payment option</title>
                      <script type="text/javascript">
                          function OnLoadEvent()
                          {
                            // Make the form post as soon as it has been loaded.
                            //document.theForm.submit();
                          }
                      </script>
                   </head>
                   <body onload="OnLoadEvent();">
                      <p>
                          If your browser does not start loading the page,
                          press the button below.
                          You will be sent back to this site after you
                          authorize the transaction.
                      </p>
                      <form name="ThreeDForm" method="POST" action="' . $authData->getUrl() . '">
                          <button class="3dsumbit" type=submit>Click Here</button>
                          <input type="hidden" name="PaReq" value="' . $authData->getData() . '" />
                          <input type="hidden" name="TermUrl" value="' . $config["url"] . 'bb-ipn.php?bb_gateway_id=' . $_REQUEST["bb_gateway_id"] . '&bb_invoice_id=' . $_REQUEST["bb_invoice_id"] . '&bb_redirect=1&bb_invoice_hash=' . $invoice->hash . '&3dreturn=3d_secure_cardinity" />
                          <input type="hidden" name="MD" value="' . $config["url"] . '" />
                      </form>
                   </body>
                </html>';
            header('Cache-Control: no-cache, must-revalidate');
            header('Expires: Mon, 26 Jul 1997 05:00:00 GMT');
            header('Content-type: application/json; charset=utf-8');
            $res = array('result' => $returnObj, 'error' => null, 'method' => '3dredirect');
            print json_encode($res);
            exit;
        }


        $tx->status = 'processed';
        $tx->updated_at = date('Y-m-d H:i:s');
        $this->di['db']->store($tx);

    }

    /**
     * @param string $url
     */
    protected function _generateForm(Model_Invoice $invoice)
    {
        $consumerKey = $this->config['consumerKey'];
        if ($this->config['test_mode']) {
            $consumerKey = $this->get_test_consumerKey();
        }

        $consumerSecret = $this->config['consumerSecret'];
        if ($this->config['test_mode']) {
            $consumerSecret = $this->get_test_consumerSecret();
        }

        $dataAmount = $this->getAmount($invoice);

        $settingService = $this->di['mod_service']('System');
        $company = $settingService->getCompany();

        $title = $this->getInvoiceTitle($invoice);

        $form = '<form action=":callbackUrl" method="POST" class="cardinity_form" data-api-redirect=":redirectUrl">';
        $form .= '<div class="cardinity_box">';
        $form .= '<label for="holder">Card Holder Name:</label>';
        $form .= '<input type="text" name="holder" id="holder" />';
        $form .= '<label for="cc_number">Cardit Card Number:</label>';
        $form .= '<input type="text" name="cc_number" id="cc_number" />';
        $form .= '</div>';
        $form .= '<div class="cardinity_box_cvv">';
        $form .= '<div class="box_exp">';
        $form .= '<label for="ex_month">Expiration Date</label>';
        $form .= '<select id="ex_month" name="ex_month" class="ex_month">';
        for ($i = 1; $i <= 12; $i++) {
            $form .= '<option value="' . $i . '">' . $i . '</option>';
        }
        $form .= '</select>';
        $form .= '<select id="ex_year" name="ex_year" class="ex_year">';
        for ($i = date('Y'); $i <= date('Y', strtotime('+20 year')); $i++) {
            $form .= '<option value="' . $i . '">' . $i . '</option>';
        }
        $form .= '</select>';
        $form .= '</div>';

        $form .= '<div class="box_cvv">';
        $form .= '<label for="cvv">CVV</label>';
        $form .= '<input type="text" name="cvv" id="cvv" />';
        $form .= '</div>';
        $form .= '</div>';

        $form .= '<div class="cardinity_submit_box">
                        <div>
                            <input class="submit" type="submit" name="sub" value=":label"/>
                        </div>
                        <div>
                            Order total: ' . number_format($dataAmount, 2) . ' :currency
                        </div>
                    </div>
                </form>
                <div id="3dform"></div>
                <style>
                .cardinity_box label{}
                .cardinity_box input{width:100%;height: 30px;}
                .cardinity_box_cvv .box_exp{width:50%;float:left;}
                .cardinity_box_cvv .box_exp select{width:48%;float:left;margin-right:5px;height:40px;}
                .cardinity_box_cvv .box_cvv{width:50%;float:left;}
                .cardinity_box_cvv .box_cvv input{width:100%;float:left;height:30px;}
                .cardinity_form input.submit{background-color: #0A77BA;color: #FFF;border: none;padding: 10px;float: right;margin-bottom: 17px;}
                .cardinity_submit_box{float:left;width:100%;     margin: 20px 0 50px 0;}
                .cardinity_submit_box div{float: right;text-align: center;height: 39px;line-height: 30px;margin-right: 18px;font-size: 16px;font-weight: bold;}
                </style>
                <script type="text/javascript">
                    $("form.cardinity_form").bind("submit", function(){
                        var redirect = $(this).attr("data-api-redirect");
                        var jsonp = $(this).attr("data-api-jsonp");
                        var msg = $(this).attr("data-api-msg");
                        var reload = $(this).attr("data-api-reload");
                        var url = $(this).attr("action");
                        if($(this).attr("data-api-url")) {
                            url = $(this).attr("data-api-url");
                        }
                        postCardinity(
                            url,
                            $(this).serialize(),
                            function(result) {
                                if(result){
                                    $("#3dform").html(result);
                                    $( document ).on( "click", "button.3dsumbit", function() {                                      
                                    });
                                    $("button.3dsumbit").click();
                                }else{
                                    if(reload !== undefined) {
                                        bb.reload();
                                        return;
                                    }
                                    if(redirect !== undefined) {
                                        bb.redirect(redirect);
                                        return;
                                    }
                                    if(msg !== undefined) {
                                        bb.msg(msg);
                                        return;
                                    }
                                    if(jsonp !== undefined && window.hasOwnProperty(jsonp)) {
                                        return window[jsonp](result);
                                    }
                                }
                            }
                        );
                        return false;
                    });
                    
                    postCardinity = function(url, params, jsonp) {
                        $.ajax({
                            type: "POST",
                            url: bb.restUrl(url),
                            data: params,
                            dataType: "json",
                            error: function(jqXHR, textStatus, e) {
                                bb.msg(e, "error");
                            },
                            success: function(data) {
                                if(data.error) {
                                    bb.msg(data.error.message, "error");
                                }else if(typeof (data.method) != "undefined" && data.method == "3dredirect"){
                                    return jsonp(data.result);
                                } else {
                                    if(typeof jsonp === "function") {
                                        return jsonp();
                                        //return jsonp(data.result);
                                    } else if(window.hasOwnProperty("console")) {
                                        console.log(data.result);
                                    }
                                }
                            }
                        });
                    }
                </script>';
        $payGatewayService = $this->di['mod_service']('Invoice', 'PayGateway');
        $payGateway = $this->di['db']->findOne('PayGateway', 'gateway = "Cardinity"');
        $bindings = array(
            ':consumerKey' => $consumerKey,
            ':consumerSecret' => $consumerSecret,
            ':amount' => $dataAmount,
            ':currency' => $invoice->currency,
            ':name' => $company['name'],
            ':description' => $title,
            ':image' => $company['logo_url'],
            ':email' => $invoice->buyer_email,
            ':label' => __('Pay now'),
            ':callbackUrl' => $payGatewayService->getCallbackUrl($payGateway, $invoice),
            ':redirectUrl' => $this->di['tools']->url('invoice/' . $invoice->hash)
        );
        return strtr($form, $bindings);
    }

    public function get_test_consumerKey()
    {
        if (!isset($this->config['testConsumerKey'])) {
            throw new Payment_Exception('Payment gateway "Cardinity" is not configured properly. Please update configuration parameter "Test Consumer Key" at "Configuration -> Payments".');
        }
        return $this->config['testConsumerKey'];
    }

    public function get_test_consumerSecret()
    {
        if (!isset($this->config['testConsumerSecret'])) {
            throw new Payment_Exception('Payment gateway "Cardinity" is not configured properly. Please update configuration parameter "Test Consumer Secret" at "Configuration -> Payments".');
        }
        return $this->config['testConsumerSecret'];
    }
}
